#!/usr/bin/python

import argparse, sys
from valid_builders import PRETTY_NAMES, DESKTOP_BUILDERS, MOBILE_BUILDERS, TEST_BUILDERS, \
                           TALOS_BUILDERS, TALOS_SUITES, UNITTEST_SUITES

'''Given a commit message or info file as a string of arguments
   returns only those builder names that should be built
   
   Comment has to be passed in split by whitespace, then split on : 
   the part before the : gets -- put in front for argparse'''

def getDesktopBuilders(platforms, customBuilderNames, buildTypes):
    if platforms == 'all':
        customBuilderNames.extend(DESKTOP_BUILDERS)
    elif platforms != 'none':
      for b in buildTypes:
        if b == 'opt':
            b = 'build'
        if b == 'debug':
            b = 'leak test build'

        for platform in platforms:
            platform = 'desktop_' + platform
            if platform in PRETTY_NAMES.keys():
                custom_builder = "%s tryserver %s" % (PRETTY_NAMES[platform], b)
                if custom_builder in DESKTOP_BUILDERS:
                    customBuilderNames.extend([custom_builder])

def getMobileBuilders(platforms, customBuilderNames):
    if platforms == 'all':
        customBuilderNames.extend(MOBILE_BUILDERS)
    elif platforms != 'none':
        for platform in platforms:
            if platform in ('win32', 'macosx', 'linux'):
                platform = 'mobile_' + platform
            if platform in PRETTY_NAMES.keys():
                custom_builder = "%s tryserver %s" % (PRETTY_NAMES[platform], b)
                if custom_builder in MOBILE_BUILDERS:
                    customBuilderNames.extend([custom_builder])

def getTestBuilders(platforms, testType, tests, customBuilderNames, buildTypes):
    # for all possible suites, add in the builderNames for that platform
    if tests == 'all':
      if testType == "test":
        tests = UNITTEST_SUITES
      if testType == "talos":
        tests = TALOS_SUITES
    if testType == "test":
        for b in buildTypes:
            for platform in platforms:
                for test in tests:
                    custom_builder = "%s tryserver %s %s %s" % (PRETTY_NAMES[platform], b, testType, test)
                    if custom_builder in (TEST_BUILDERS):
                        customBuilderNames.extend([custom_builder])

    if testType == "talos":
        for platform in platforms:
            for test in tests:
                custom_builder = "%s tryserver %s %s" % (PRETTY_NAMES[platform], testType, test)
                if custom_builder in (TALOS_BUILDERS):
                    customBuilderNames.extend([custom_builder])

def tryParser(message, customBuilderNames):
    parser = argparse.ArgumentParser(description='Pass in a commit message and a list \
                                     and tryParse populates the list with the builderNames\
                                     that need schedulers.')
    parser.add_argument('--try',
                        default='d',
                        dest='build_type',
                        choices='bdo',
                        help='accepts the build types requested ')
    parser.add_argument('--p',
                        default='all',
                        dest='desktop',
                        help='provide a list of desktop platforms, or specify none (default is all)')
    parser.add_argument('--m',
                        default='all',
                        dest='mobile',
                        help='provide a list of mobile platform, or specify none (default is all)')
    parser.add_argument('--u',
                        default='none',
                        dest='test',
                        help='provide a list of unit tests, or specify all (default is None)')
    parser.add_argument('--t',
                        default='none',
                        dest='talos',
                        help='provide a list of talos tests, or specify all (default is None)')
    parser.add_argument('builderNames',
                        help='list of builder names')

    (args, extra) = parser.parse_known_args(message)
    print args.build_type
    # Build Types input validation is covered by choices
    if args.build_type == 'b':
        args.build_type = ['opt', 'debug']
    elif args.build_type == 'd':
        args.build_type = ['debug']
    elif args.build_type == 'o':
        args.build_type = ['opt']

    # Get the custom builder names
    getDesktopBuilders(args.desktop, customBuilderNames, args.build_type)
    getMobileBuilders(args.mobile, customBuilderNames)
    getTestBuilders(args.desktop, "test", args.test, customBuilderNames, args.build_type)
    getTestBuilders(args.desktop, "talos", args.talos, customBuilderNames, args.build_type)

if __name__ == "__main__":
    message = ['--try', 'b', '--p', ['linux','macosx'], '--u', ['reftest'], '--t', 'all', ['linux-talos','linux-unittest', 'macosx-talos']]
    customBuilderNames = []
    tryParser(message, customBuilderNames)
    print customBuilderNames